from unittest import TestCase
from mock import patch

from nose.tools import assert_equal


mock_bower = patch('django_collectstatic_bower.bower.bower')


class FinderTests(TestCase):

	def setUp(self):
		from django_collectstatic_bower.staticfiles.finders import BowerComponentFinder
		self.finder = BowerComponentFinder()

	@mock_bower
	def test_finder_runs_bower_install(self, bower_mock):
		"""Verify bower install is done before finding or collecting static files"""

		self.finder.list([])

		self.finder.find('')

		assert_equal(bower_mock.call_count, 2)
		bower_mock.assert_called_with('install')




import json
import os

import subprocess
from django.conf import settings


def which(cmd, mode=os.F_OK | os.X_OK, path=None):
	"""Given a command, mode, and a PATH string, return the path which
	conforms to the given mode on the PATH, or None if there is no such
	file.

	`mode` defaults to os.F_OK | os.X_OK. `path` defaults to the result
	of os.environ.get("PATH"), or can be overridden with a custom search
	path.

	"""
	# Check that a given file can be accessed with the correct mode.
	# Additionally check that `file` is not a directory, as on Windows
	# directories pass the os.access check.
	def _access_check(fn, mode):
		return (os.path.exists(fn) and os.access(fn, mode)
				and not os.path.isdir(fn))

	# If we're given a path with a directory part, look it up directly rather
	# than referring to PATH directories. This includes checking relative to the
	# current directory, e.g. ./script
	if os.path.dirname(cmd):
		if _access_check(cmd, mode):
			return cmd
		return None

	if path is None:
		path = os.environ.get("PATH", os.defpath)
	if not path:
		return None
	path = path.split(os.pathsep)

	if sys.platform == "win32":
		# The current directory takes precedence on Windows.
		if not os.curdir in path:
			path.insert(0, os.curdir)

		# PATHEXT is necessary to check on Windows.
		pathext = os.environ.get("PATHEXT", "").split(os.pathsep)
		# See if the given file matches any of the expected path extensions.
		# This will allow us to short circuit when given "python.exe".
		# If it does match, only test that one, otherwise we have to try
		# others.
		if any(cmd.lower().endswith(ext.lower()) for ext in pathext):
			files = [cmd]
		else:
			files = [cmd + ext for ext in pathext]
	else:
		# On other platforms you don't have things like PATHEXT to tell you
		# what file suffixes are executable, so just pass on cmd as-is.
		files = [cmd]

	seen = set()
	for dir in path:
		normdir = os.path.normcase(dir)
		if not normdir in seen:
			seen.add(normdir)
			for thefile in files:
				name = os.path.join(dir, thefile)
				if _access_check(name, mode):
					return name
	return None


def get_bower_executable_path():
	"""Find bower executable"""
	try:
		return getattr(settings, 'BOWER_CMD')
	except AttributeError:
		return which('bower')


def get_bower_rc_path():
	"""Find .bowerrc file"""
	return getattr(settings, 'BOWER_RC_FILE', os.path.join(
		os.getcwd(),
		'.bowerrc'
	))


def get_bower_components_path():
	"""Returns bower components path"""
	rc = load_bower_rc()

	if rc:
		path = os.path.abspath(rc.get('directory', 'bower_components/'))
	else:
		path = os.path.join(
			os.getcwd(),
			'bower_components/'
		)

	return os.path.abspath(path)


def load_bower_rc():
	"""Returns parsed .bowerrc file"""
	try:
		with open(get_bower_rc_path()) as rc:
			return json.load(rc)
	except Exception:
		return None


def bower(*args):
	"""Runs bower with provided args"""
	cwd = os.getcwd()

	proc = subprocess.Popen(
		(get_bower_executable_path(),) + args,
		cwd=cwd
	)
	proc.wait()

	if proc.returncode:
		raise RuntimeError('Bower command failed')

	return proc



"""Tests bower adapter"""
import os
import mock

from django.test import override_settings
from nose.tools import assert_equal

from django_collectstatic_bower import bower


@mock.patch('django_collectstatic_bower.bower.get_bower_executable_path')
@mock.patch('subprocess.Popen')
def test_bower_subprocess(subprocess_mock, exe_path_mock):
	"""Tests that the provided bower executable is launched in a subprocess"""
	exe_path = '/path/to/bower'
	cmd = 'install'
	cwd = os.getcwd()

	exe_path_mock.return_value = exe_path
	proc_mock = mock.Mock()
	proc_mock.configure_mock(**{
		'wait': lambda: None,
		'returncode': 0
	})
	subprocess_mock.return_value = proc_mock

	bower.bower(cmd)

	subprocess_mock.assert_called_with((exe_path, cmd), cwd=cwd)


def test_bower_get_executable():
	"""Tests that the get_bower_cmd() finds the path to the bower executable"""
	# Verify django settings override option
	with override_settings(BOWER_CMD='/override/path'):
		assert_equal(bower.get_bower_executable_path(), '/override/path')

	with override_settings():
		with mock.patch('django_collectstatic_bower.bower.which') as which:
			which.return_value = '/discovered/path'
			assert_equal(bower.get_bower_executable_path(), '/discovered/path')


def test_bower_get_rc_file():
	"""Tests that the .bowerrc file is discoverable and usable as a JSON object"""
	with override_settings(BOWER_RC_FILE='/path/to/rc'):
		assert_equal(bower.get_bower_rc_path(), '/path/to/rc')

	with override_settings():
		assert_equal(bower.get_bower_rc_path(), os.getcwd() + '/.bowerrc')


def test_bower_get_component_path():
	"""Tests that the directory bower installs to is discovered correctly"""

	with mock.patch('django_collectstatic_bower.bower.load_bower_rc') as rc:
		rc.return_value = {'directory': '/path/to/components'}

		assert_equal(bower.get_bower_components_path(), '/path/to/components')



"""Django settings module for test environment"""
import os

INSTALLED_APPS = (
	'django_collectstatic_bower',
	'django.contrib.staticfiles',
)

SECRET_KEY = os.urandom(40)

STATIC_URL = '/static/'
STATIC_ROOT = os.path.join(os.getcwd(), 'static')

MEDIA_URL = '/media/'



from django.contrib.staticfiles.finders import BaseStorageFinder
from django.core.files.storage import FileSystemStorage

from django_collectstatic_bower import bower


class BowerComponentFinder(BaseStorageFinder):
	"""Finder that returns bower components, installing them if necessary"""

	storage = FileSystemStorage(bower.get_bower_components_path())

	def install_components(self):
		bower.bower('install')

	def find(self, path, all=False):
		self.install_components()

		return super(BowerComponentFinder, self).find(path, all)

	def list(self, ignore_patterns):
		"""Installs bower components listed in bower.json, then lists all installed files"""
		self.install_components()

		return super(BowerComponentFinder, self).list(ignore_patterns)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

from django_collectstatic_bower import __version__, __doc__

def parse_requirements(requirements):
	with open(requirements) as f:
		return [l.strip('\n') for l in f if l.strip('\n') and not l.startswith('#')]

with open('README.rst') as readme:
	long_description = readme.read()

test_requirements = parse_requirements('test-requirements.txt')

setup(
	name='django-collectstatic-bower',
	version=__version__,
	packages=find_packages(exclude=('tests.*', 'tests',)),
	url='https://github.com/nick-allen/django-collectstatic-bower',
	license='MIT',
	author='Nick Allen',
	author_email='nick.allen.cse@gmail.com',
	description=__doc__,
	long_description=long_description,
	include_package_data=True,
	zip_safe=False,
	install_requires=parse_requirements('requirements.txt'),
	extras_require={
		'test': test_requirements
	},
	test_suite='nose.collector',
	tests_require=test_requirements
)
